dnl -*- m4 -*-
dnl----------------------------------------------------------------------
dnl These are custom rules for certain programs
dnl----------------------------------------------------------------------
dnl======================================================================
dnl LDAS_PROG_BISON
dnl   Check on how to setup bison
dnl Depends:
dnl======================================================================
AC_DEFUN([LDAS_PROG_BISON],
[ AC_REQUIRE([AC_PROG_YACC])
  case "$YACC" in
  bison\ *) YACC="bison --debug --locations";;
  esac
])

dnl======================================================================
dnl AC_LDAS_PROG_CC
dnl   Check on how to setup cc
dnl Depends:
dnl======================================================================

AC_DEFUN([AC_LDAS_PROG_CC],
[ AC_REQUIRE([AC_PROG_CC])
  if test x${ac_cv_prog_cc_g} = xyes
  then
    dnl------------------------------------------------------------------
    dnl PR3112: Put in -g (debug) option if supported by compiler
    dnl------------------------------------------------------------------
    CFLAGS="-g $CFLAGS"
  fi
  if test "${GCC}" = "yes"
  then
    touch ,t.c
    ac_ldas_default_cc_includes="`${CC} -v ,t.c 2>&1 | \
  	sed -n -e '/\#include/,/End of search list\./p' | \
  	egrep '^ '`"
    rm ,t.c
  fi
  #=======================================================================
  # Determine compiler vendor
  #=======================================================================
  AC_LANG_SAVE
  AC_LANG_C
  LDAS_CC_FLAVOR="generic"
  ldas_prog_cc_vendor=generic
  if test $ldas_prog_cc_vendor = generic
  then
    dnl -----------------------------------------------------------------
    dnl  Checking if GNU flavor
    dnl -----------------------------------------------------------------
    if test "${GCC}" = yes
    then
      ldas_prog_cc_vendor="GNU"
      LDAS_CC_FLAVOR=gnu
    fi
  fi
  if test $ldas_prog_cc_vendor = generic
  then
    dnl -----------------------------------------------------------------
    dnl  Checking if SunPRO flavor
    dnl -----------------------------------------------------------------
    AC_TRY_RUN([ #include <stdlib.h>

                 int main( );

                 int
                 main( )
                 {
                   #if __SUNPRO_C
                   exit( 0 );
                   #endif /* __SUNPRO_C */
	           exit( 1 );
                 }
               ],[
                ldas_prog_cc_vendor=Sun
        	LDAS_CC_FLAVOR="SunPRO"
               ])
  fi
  AC_LANG_RESTORE
  AM_CONDITIONAL([LDAS_CC_SUNPRO],[test x$LDAS_CC_FLAVOR = xSunPRO])
  dnl -------------------------------------------------------------------
  dnl  setting up of compiler flags
  dnl -------------------------------------------------------------------
  AC_MSG_CHECKING(C vendor)
  AC_MSG_RESULT($ldas_prog_cc_vendor)
  case x$ldas_prog_cc_vendor in
  xGNU) dnl ######## G N U ########
    dnl -----------------------------------------------------------------
    dnl  Optimization
    dnl -----------------------------------------------------------------
    ldas_prog_cc_optimization_key="-O"
    ldas_prog_cc_optimization_none="-O0"
    ldas_prog_cc_optimization_exrtreme="-O4"
    ldas_prog_cc_optimization_high="-O3"
    ldas_prog_cc_optimization_medium="-O2"
    ldas_prog_cc_optimization_low="-O1"
    ldas_prog_cc_optimization_default=""
    dnl -----------------------------------------------------------------
    dnl   64 bit
    dnl -----------------------------------------------------------------
    case $ldas_processor in
    sparc)
      ldas_prog_cc_64bit="-m64 -mcpu=v9"
      ;;
    *)
      ldas_prog_cc_64bit="-m64"
      ;;
    esac
    dnl -----------------------------------------------------------------
    dnl  Handling of warnings
    dnl -----------------------------------------------------------------
    ldas_prog_cc_warning="-Wall -Wextra -Wno-missing-field-initializers -Wno-unused-parameter"
    ldas_prog_cxx_warning="-Wall -Wextra -Wno-missing-field-initializers -Wno-unused-parameter -Wignored-qualifiers"
    ldas_prog_cc_warning_as_error="-Werror"
    ldas_prog_cxx_warning_as_error="-Werror"
    ;;
  xSun) dnl ######## S U N   S T U D I O ########
    dnl -----------------------------------------------------------------
    dnl  Optimization
    dnl -----------------------------------------------------------------
    ldas_prog_cc_optimization_key="-O"
    ldas_prog_cc_optimization_none="-xO0"
    ldas_prog_cc_optimization_exrtreme="-xO5"
    ldas_prog_cc_optimization_high="-xO4"
    ldas_prog_cc_optimization_medium="-xO3"
    ldas_prog_cc_optimization_low="-xO1"
    ldas_prog_cc_optimization_default=""
    dnl -----------------------------------------------------------------
    dnl   64 bit
    dnl -----------------------------------------------------------------
    ldas_prog_cc_64bit="-xarch=generic64"
    dnl -----------------------------------------------------------------
    dnl  Handling of warnings
    dnl -----------------------------------------------------------------
    ldas_prog_cc_warning=""
    ldas_prog_cxx_warning=""
    ldas_prog_cc_warning_as_error="-errwarn=%all"
    ldas_prog_cxx_warning_as_error="-errwarn=%all"
    ;;
  *) dnl ###### D E F A U L T ########
    dnl -----------------------------------------------------------------
    dnl  Optimization
    dnl -----------------------------------------------------------------
    ldas_prog_cc_optimization_key="-O"
    ldas_prog_cc_optimization_none="-O0"
    ldas_prog_cc_optimization_exrtreme="-O4"
    ldas_prog_cc_optimization_high="-O3"
    ldas_prog_cc_optimization_medium="-O2"
    ldas_prog_cc_optimization_low="-O1"
    ldas_prog_cc_optimization_default=""
    ;;
  esac
  dnl -------------------------------------------------------------------
  dnl Setup flags for 64 bit compilation
  dnl -------------------------------------------------------------------
  AC_REQUIRE([AC_LDAS_ARG_ENABLE_64BIT])
  case x${enable_64bit} in
  xyes)
    CFLAGS="$CFLAGS $ldas_prog_cc_64bit"
    ;;
  esac
  dnl -------------------------------------------------------------------
  dnl  Checking on how to pass rpath info
  dnl -------------------------------------------------------------------
  saved_LDFLAGS="$LDFLAGS"
  LDFLAGS="$LDFLAGS -Wl,--rpath,/usr/lib"
  AC_TRY_RUN([ #include <stdlib.h>

               int main( );

               int
               main( )
               {
                   exit( 0 );
               }
             ],[
	        RPATH="--rpath"
             ])
  LDFLAGS="$saved_LDFLAGS"
  case "$RPATH" in
  "")
    saved_LDFLAGS="$LDFLAGS"
    LDFLAGS="$LDFLAGS -Wl,-R,/usr/lib"
    AC_TRY_RUN([ #include <stdlib.h>

                 int main( );

                 int
                 main( )
                 {
                   exit( 0 );
                 }
               ],[
	        RPATH="-R"
               ])
    LDFLAGS="$saved_LDFLAGS"
    ;;
  esac
])

dnl======================================================================
dnl AC_LDAS_PROG_CXX
dnl   Check on how to setup c++
dnl Depends:
dnl======================================================================

AC_DEFUN([AC_LDAS_PROG_CXX],
[ AC_REQUIRE([AC_PROG_CXX])
  if test x${ac_cv_prog_cxx_g} = xyes
  then
    dnl------------------------------------------------------------------
    dnl PR3112: Put in -g (debug) option if supported by compiler
    dnl------------------------------------------------------------------
    CXXFLAGS="-g $CXXFLAGS"
  fi
  if test "${GXX}" = "yes"
  then
    touch ,t.cc
    ac_default_cxx_includes="`${CXX} -v ,t.cc 2>&1 | \
  	sed -n -e '/\#include/,/End of search list\./p' | \
  	egrep '^ '`"
    rm ,t.cc
  fi
  #=======================================================================
  # Determine compiler vendor
  #=======================================================================
  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS
  LDAS_CXX_FLAVOR="generic"
  ldas_prog_cxx_vendor=generic
  if test $ldas_prog_cxx_vendor = generic
  then
    dnl -----------------------------------------------------------------
    dnl  Checking if GNU flavor
    dnl -----------------------------------------------------------------
    if test "${GXX}" = yes
    then
      ldas_prog_cxx_vendor="GNU"
      LDAS_CXX_FLAVOR=gnu
    fi
  fi
  if test $ldas_prog_cxx_vendor = generic
  then
    dnl -----------------------------------------------------------------
    dnl  Checking if SunPRO flavor
    dnl -----------------------------------------------------------------
    AC_TRY_RUN([ #include <stdlib.h>

                 int main( );

                 int
                 main( )
                 {
                   #if __SUNPRO_CC
                   exit( 0 );
                   #endif /* __SUNPRO_CC */
	           exit( 1 );
                 }
               ],[
                ldas_prog_cxx_vendor=Sun
        	LDAS_CXX_FLAVOR="SunPRO"
               ])
  fi
  AC_LANG_RESTORE
  dnl -------------------------------------------------------------------
  dnl  setting up of compiler flags
  dnl -------------------------------------------------------------------
  AC_MSG_CHECKING(C++ vendor)
  AC_MSG_RESULT($ldas_prog_cxx_vendor)
  case x$ldas_prog_cxx_vendor in
  xGNU) dnl ######## G N U ########
    dnl -----------------------------------------------------------------
    dnl  Optimization
    dnl -----------------------------------------------------------------
    ldas_prog_cxx_optimization_key="-O"
    ldas_prog_cxx_optimization_none="-O0"
    ldas_prog_cxx_optimization_exrtreme="-O4"
    ldas_prog_cxx_optimization_high="-O3"
    ldas_prog_cxx_optimization_medium="-O2"
    ldas_prog_cxx_optimization_low="-O1"
    ldas_prog_cxx_optimization_default=""
    dnl -----------------------------------------------------------------
    dnl   64 bit
    dnl -----------------------------------------------------------------
    case $ldas_processor in
    sparc)
      ldas_prog_cxx_64bit="-m64 -mcpu=v9"
      ;;
    *)
      ldas_prog_cxx_64bit="-m64"
      ;;
    esac
    dnl -----------------------------------------------------------------
    dnl  Handling of warnings
    dnl -----------------------------------------------------------------
    ldas_prog_cxx_warning="-Wall -Wextra -Wno-missing-field-initializers -Wno-unused-parameter"
    ldas_prog_cxx_warning_as_error="-Werror"
    ;;
  xSun) dnl ######## S U N   S T U D I O ########
    dnl -----------------------------------------------------------------
    dnl  Optimization
    dnl -----------------------------------------------------------------
    ldas_prog_cxx_optimization_key="-O"
    ldas_prog_cxx_optimization_none="-xO0"
    ldas_prog_cxx_optimization_exrtreme="-xO4"
    ldas_prog_cxx_optimization_high="-xO3"
    ldas_prog_cxx_optimization_medium="-xO2"
    ldas_prog_cxx_optimization_low="-xO1"
    ldas_prog_cxx_optimization_default=""
    dnl -----------------------------------------------------------------
    dnl   64 bit
    dnl -----------------------------------------------------------------
    ldas_prog_cxx_64bit="-m64"
    dnl -----------------------------------------------------------------
    dnl  Handling of warnings
    dnl -----------------------------------------------------------------
    ldas_prog_cxx_warning=""
    ldas_prog_cxx_warning_as_error="-errwarn=%all"
    dnl -----------------------------------------------------------------
    dnl  Setting up of CXXFLAGS
    dnl -----------------------------------------------------------------
    dnl Use STLport, not default
    CXXFLAGS="$CXXFLAGS -library=stlport4"
    dnl AC_CHECK_LIB([sunmath],[coshf])
    dnl -----------------------------------------------------------------
    dnl  Checking Version of Sun Studio compiler
    dnl -----------------------------------------------------------------
    AC_LANG_SAVE
    AC_LANG_CPLUSPLUS
    AC_TRY_RUN([ #include <stdlib.h>

                 int main( );

                 int
                 main( )
                 {
                   #if __SUNPRO_CC <= 0x0590
                   exit( 0 );
                   #endif /* __SUNPRO_CC */
	           exit( 1 );
                 }
               ],[ldas_inline_kludge="inline"
               ])
    AC_LANG_RESTORE
    ;;
  *) dnl ###### D E F A U L T ########
    dnl -----------------------------------------------------------------
    dnl  Optimization
    dnl -----------------------------------------------------------------
    ldas_prog_cxx_optimization_key="-O"
    ldas_prog_cxx_optimization_none="-O0"
    ldas_prog_cxx_optimization_exrtreme="-O4"
    ldas_prog_cxx_optimization_high="-O3"
    ldas_prog_cxx_optimization_medium="-O2"
    ldas_prog_cxx_optimization_low="-O1"
    ldas_prog_cxx_optimization_default=""
    ;;
  esac
  AM_CONDITIONAL([LDAS_CXX_SUNPRO],[test x$LDAS_CXX_FLAVOR = xSunPRO])

  dnl -------------------------------------------------------------------
  dnl Setup flags for 64 bit compilation
  dnl -------------------------------------------------------------------
  AC_REQUIRE([AC_LDAS_ARG_ENABLE_64BIT])
  case x${enable_64bit} in
  xyes)
    CXXFLAGS="$CXXFLAGS $ldas_prog_cxx_64bit"
    ;;
  esac
  AC_DEFINE_UNQUOTED([INLINE_KLUDGE],${ldas_inline_kludge},Set to "inline" if need to inline kludge to prevent multi/undefined variables)
  dnl -------------------------------------------------------------------
  dnl  Perform all tests using C++
  dnl -------------------------------------------------------------------
  AC_LANG_CPLUSPLUS
])

dnl======================================================================
dnl LDAS_PROG_DOT
dnl======================================================================
AC_DEFUN([LDAS_PROG_DOT],
[ AC_MSG_CHECKING(for dot)
  AC_PATH_PROGS( DOT, dot )
  case x$DOT in
  x) AC_MSG_RESULT(no);;
  *)
    AC_MSG_RESULT($DOT)
    DOT_PATH="`echo $DOT | sed -e 's,/[^/]*$,,'`"
    ;;
  esac
  AC_SUBST(DOT_PATH)
  AM_CONDITIONAL(HAVE_DOT, test x$DOT != x)
])

dnl======================================================================
dnl LDAS_PROG_DOXYGEN
dnl======================================================================
AC_DEFUN([LDAS_PROG_DOXYGEN],
[ AC_REQUIRE([LDAS_PROG_LATEX])
  AC_REQUIRE([LDAS_PROG_DOT])
  AC_ARG_WITH([doxygen],
	[  --with-doxygen=<PATH>  Specify the path for doxygen],
	[],[with_doxygen=doxygen])
  case x"$with_doxygen" in
  x|xno)
    DOXYGEN=""
    ;;
  * )
    AC_MSG_CHECKING(for doxygen)
    AC_PATH_PROGS( DOXYGEN, doxygen )
    case x$DOXYGEN in
    x) AC_MSG_RESULT(no);;
    *) AC_MSG_RESULT($DOXYGEN);;
    esac
  esac
  AM_CONDITIONAL(HAVE_DOXYGEN, test x$DOXYGEN != x)
])

dnl======================================================================
dnl LDAS_PROG_LATEX
dnl======================================================================
AC_DEFUN([LDAS_PROG_LATEX],
[ AC_MSG_CHECKING(for latex)
  AC_PATH_PROGS( LATEX, latex )
  case x$LATEX in
  x) AC_MSG_RESULT(no);;
  *) AC_MSG_RESULT($LATEX);;
  esac
  AM_CONDITIONAL(HAVE_LATEX, test x$LATEX != x)
])

dnl======================================================================
dnl LDAS_PROG_LEX
dnl   Check on how to setup lex
dnl Depends:
dnl======================================================================
AC_DEFUN([LDAS_PROG_LEX],
[ AM_PROG_LEX
  case x$LEX in
  x/*)
    ;;
  *)
    AC_PATH_PROGS(LEX,flex lex)
    ;;
  esac
  case x$LEX in
  */flex|*/flex++)
    dnl------------------------------------------------------------------
    dnl If the version of flex installed on the destination machine
    dnl   is different from what generated the sources, then
    dnl   remove the sources so they will be regenerated.
    dnl------------------------------------------------------------------
    AC_EGREP_CPP([yy_current_buffer],
    [ #include <FlexLexer.h>
    ],
    [ AC_DEFINE([HAVE_YY_CURRENT_BUFFER],1,[Defined if yy_current_buffer is in yyFlexLexer class])
    ],
    [ AC_DEFINE([YY_CURRENT_BUFFER_EQUIV],[(yy_buffer_stack[[yy_buffer_stack_top]])],[Defined as the equivelent fro yy_current_buffer])
    ])
    ;;
  esac
])

dnl======================================================================
dnl AC_LDAS_PROG_LIBTOOL
dnl   Check on how to setup libtool
dnl Depends:
dnl======================================================================

AC_DEFUN([AC_LDAS_PROG_LIBTOOL],
[
  dnl -------------------------------------------------------------------
  dnl Need to ensure the correct usage of the C compiler
  dnl -------------------------------------------------------------------
  AC_REQUIRE([AC_LDAS_PROG_CC])
  dnl -------------------------------------------------------------------
  dnl This hack is needed until libtool 1.5.22 is out. It will properly
  dnl    configure for 64bit Solaris.
  dnl -------------------------------------------------------------------
  ac_save_CC=$CC
  ac_save_CXX=$CXX
  CC="$CC $CFLAGS"
  CXX="$CXX $CXXFLAGS"
  AC_LANG_SAVE
  AC_LANG_C
  AC_PROG_LIBTOOL
  AM_PROG_LIBTOOL
  AC_LANG_RESTORE
  case "$LDAS_CXX_FLAVOR" in
  SunPRO)
	if echo $CXXFLAGS | grep -e -library=stlport4 > /dev/null 2>&1
	then
		dnl -----------------------------------------------------
		dnl libtool currently assumes the default std C++ library.
		dnl   LDAS requires the stlport4 library instead.
		dnl -----------------------------------------------------
		mv libtool libtool.orig
		sed -e 's/-lCstd -lCrun/-library=stlport4/g' libtool.orig > libtool
		chmod +x libtool
	fi
	;;
  esac
  CC=$ac_save_CC
  CXX=$ac_save_CXX
])

dnl======================================================================
dnl AC_LDAS_PROG_PCE2
dnl   Check on how to setup pce2
dnl======================================================================

AC_DEFUN([AC_LDAS_PROG_PCE2],
[ AC_REQUIRE([LDAS_PROG_DOXYGEN])
  dnl if test x$DOXYGEN = x
  dnl then
    dnl =============================================================
    dnl  Use the much older perceps package only if doxygen is
    dnl    not available.
    dnl =============================================================
    if test -x ${srcdir}/${ldas_top}/lib/perceps/pce2
    then
      PERCEPS='[$][(]ldas_top_srcdir[)]/lib/perceps/pce2'
    elif test -x ${prefix}/bin/pce2
    then
      PERCEPS='[$][(]bindir[)]/pce2'
    fi
    if test -d ${srcdir}/${ldas_top}/lib/perceps
    then
      PERCEPS_ENV='[$][(]ldas_top_srcdir[)]/lib/perceps'
    elif test -d "${prefix}/lib/perceps"
    then
      PERCEPS_ENV='[$][(]libdir[)]/perceps'
    else
      PERCEPS_ENV=""
    fi
  dnl fi
  if test -z "${PERCEPS}" \
	|| test -z "${PERCEPS_ENV}"
  then
    dnl =============================================================
    dnl  Reset variables to be empty since there is no perceps.
    dnl =============================================================
    PERCEPS="";
    PERCEPS_FLAGS=""
    PERCEPS_ENV=""
    HAVE_PERCEPS="no"
  else
    HAVE_PERCEPS="yes"
  fi
  AC_SUBST(PERCEPS)
  AC_SUBST(PERCEPS_FLAGS)
  AC_SUBST(PERCEPS_ENV)
  AC_SUBST(HAVE_PERCEPS)
  AM_CONDITIONAL(LDAS_COND_HAVE_PERCEPS, test x$PERCEPS != x)
  PERCEPS_STRIP_GENERATING="sed -e 's/^Generating //' -e 's/^\.\/\///'"
  AC_SUBST(PERCEPS_STRIP_GENERATING)
])

dnl======================================================================
dnl AC_LDAS_PROG_PERCEPS
dnl   Check on how to setup perceps
dnl======================================================================

AC_DEFUN([AC_LDAS_PROG_PERCEPS],
[ AC_PATH_PROG(PERCEPS, perceps)
  PERCEPS_FLAGS="-abhmr"
  if test -d ${srcdir}/${ldas_top}/lib/perceps
  then
    PERCEPS_ENV='[$][(]ldas_top_srcdir[)]/lib/perceps'
  elif test -d ${prefix}/lib/perceps
  then
    PERCEPS_ENV="${prefix}/lib/perceps"
  else
    PERCEPS_ENV=""
  fi
  if test -z "${PERCEPS}" \
	|| test -d "${PERCEPS}" || test ! -x "${PERCEPS}" \
	|| test -z "${PERCEPS_ENV}"
  then
    dnl =============================================================
    dnl  Reset variables to be empty since there is no perceps.
    dnl =============================================================
    PERCEPS="";
    PERCEPS_FLAGS=""
    PERCEPS_ENV=""
    HAVE_PERCEPS="no"
  else
    HAVE_PERCEPS="yes"
    PERCEPS_FLAGS=""
  fi
  AC_SUBST(PERCEPS_FLAGS)
  AC_SUBST(PERCEPS_ENV)
  AC_SUBST(HAVE_PERCEPS)
  AM_CONDITIONAL(HAVE_PERCEPS, test x$PERCEPS != x)
  PERCEPS_STRIP_GENERATING="sed -e 's/^Generating //' -e 's/^\.\/\//'"
  AC_SUBST(PERCEPS_STRIP_GENERATING)
])

dnl======================================================================
dnl LDAS_PROG_SWIG
dnl   Check on how to setup swig
dnl======================================================================

AC_DEFUN([LDAS_PROG_SWIG],
[ AC_PATH_PROG( SWIG, swig )
  case "`$SWIG -swiglib`" in
  *swig_lib) ;;
  *) SWIGFLAGS="$SWIGFLAGS -DSWIGNEW";;
  esac
  AC_SUBST(SWIGFLAGS)
  AC_SUBST(SWIG)
  AM_CONDITIONAL(HAVE_SWIG, test x$SWIG != x)
])

dnl======================================================================
dnl LDAS_PROG_VALGRIND
dnl======================================================================
AC_DEFUN([LDAS_PROG_VALGRIND],
[ AC_PATH_PROGS( VALGRIND, valgrind )
  AM_CONDITIONAL(HAVE_VALGRIND, test x$VALGRIND != x)
])
