#ifndef FrameCPP_VERSION_6_FrProcData_HH
#define FrameCPP_VERSION_6_FrProcData_HH

#if ! defined(SWIGIMPORTED)
#include <memory>
#include <vector>
#endif /* ! defined(SWIGIMPORTED) */

#if ! defined(SWIGIMPORTED)
#include "framecpp/Common/Array.hh"
#include "framecpp/Common/Container.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version4/FrProcData.hh"
#endif /* ! defined(SWIGIMPORTED) */

#include "framecpp/Version6/FrameSpec.hh"
#if ! defined(SWIGIMPORTED)
#include "framecpp/Version6/Dimension.hh"
#include "framecpp/Version6/FrHistory.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/STRING.hh"
#endif /* ! defined(SWIGIMPORTED) */

namespace FrameCPP
{
  namespace Version_6
  {
    
    //===================================================================
    // Data elements for an FrProcData
    //===================================================================
    class FrProcDataData
    {
    public:
      typedef INT_2U	type_type;
      typedef INT_2U 	subType_type;
      typedef REAL_8	timeOffset_type;
      typedef REAL_8	tRange_type;
      typedef REAL_8	fShift_type;
      typedef REAL_4	phase_type;
      typedef REAL_8	fRange_type;
      typedef REAL_8	BW_type;

      enum e_type_type {
        UNKNOWN_TYPE = 0,
        TIME_SERIES = 1,
        FREQUENCY_SERIES = 2,
        OTHER_1D_SERIES_DATA = 3,
        TIME_FREQUENCY = 4,
        WAVELETS = 5,
        MULTI_DIMENSIONAL = 6
      };

      enum e_subType_type {
        UNKNOWN_SUB_TYPE = 0,
        //---------------------------------------------------------------
        // Subtype for fSeries
        //---------------------------------------------------------------
        DFT = 1,
        AMPLITUDE_SPECTRAL_DENSITY = 2,
        POWER_SPECTRAL_DENSITY = 3,
        CROSS_SPECTRAL_DENSITY = 4,
        COHERENCE = 5,
        TRANSFER_FUNCTION = 6
      };

      struct AuxParam_type {
        typedef REAL_8 auxParam_type;
        typedef STRING auxParamName_type;

        auxParam_type   	auxParam;
        auxParamName_type       auxParamName;

        inline
        AuxParam_type( )
        {
        };

        inline
        AuxParam_type( auxParam_type Param, auxParamName_type ParamName )
          : auxParam( Param ),
            auxParamName( ParamName )
        {
        };

        AuxParam_type( auxParamName_type ParamName, auxParam_type Param )
          : auxParam( Param ),
            auxParamName( ParamName )
        {
        };

        cmn_streamsize_type Bytes( const Common::StreamBase& Stream ) const;

        bool operator==( const AuxParam_type& RHS ) const;
      };

      typedef Common::Array< INT_2U, AuxParam_type >      AuxParamList_type;

      static const phase_type PHASE_UNKNOWN;
      static const fRange_type FRANGE_UNKNOWN;
      static const BW_type BW_UNKNOWN;

      FrProcDataData( );
      FrProcDataData( const Previous::FrProcDataData& Source );
      FrProcDataData( Common::IStream& Stream );

      // Mutators
      void AppendComment( const std::string& comment );

      const std::string& GetName( ) const; 
      const std::string& GetComment( ) const;
      type_type GetType( ) const;
      subType_type GetSubType( ) const;
      timeOffset_type GetTimeOffset( ) const;
      tRange_type GetTRange( ) const;
      fShift_type GetFShift( ) const;
      phase_type GetPhase( ) const;
      fRange_type GetFRange( ) const;
      BW_type GetBW( ) const;
      const AuxParamList_type& GetAuxParam( ) const;
      AuxParamList_type& GetAuxParam( );

      void SetFRange( REAL_8 FRange );
      void SetFShift( REAL_8 FShift );
      void SetTimeOffset( REAL_8 time );
      void SetTRange( REAL_8 TRange );

    protected:

      struct fr_proc_data_data_type {
	STRING			name;
	STRING			comment;
	type_type		type;
	subType_type		subType;
	timeOffset_type		timeOffset;
	tRange_type		tRange;
	fShift_type		fShift;
	phase_type		phase;
	fRange_type		fRange;
	BW_type			BW;
	AuxParamList_type	auxParams;

	fr_proc_data_data_type( );
	fr_proc_data_data_type( const fr_proc_data_data_type& Source );
	fr_proc_data_data_type( Common::IStream& Stream );

	/// \brief assignment operator
	const fr_proc_data_data_type&
	operator=( const fr_proc_data_data_type& Source );

	//: Comparison operator
	bool operator==( const fr_proc_data_data_type& RHS ) const;
	bool operator!=( const fr_proc_data_data_type& RHS ) const;

	void operator( )( Common::IStream& Stream );
	void operator( )( Common::OStream& Stream ) const;
      };

      mutable fr_proc_data_data_type	m_data;
    };

    inline void FrProcDataData::
    AppendComment( const std::string& comment )
    {
      FrameCPP::Common::AppendComment( m_data.comment, comment );
    } 

    inline const FrProcDataData::AuxParamList_type& FrProcDataData::
    GetAuxParam( ) const
    {
      return m_data.auxParams;
    }

    inline FrProcDataData::AuxParamList_type& FrProcDataData::
    GetAuxParam( )
    {
      return m_data.auxParams;
    }

    inline const std::string& FrProcDataData::
    GetName( ) const
    {
      return m_data.name;
    }

    inline const std::string& FrProcDataData::
    GetComment( ) const
    {
      return m_data.comment;
    }

    inline FrProcDataData::type_type FrProcDataData::
    GetType( ) const
    {
      return m_data.type;
    }

    inline FrProcDataData::subType_type FrProcDataData::
    GetSubType( ) const
    {
      return m_data.subType;
    }

    inline FrProcDataData::timeOffset_type FrProcDataData::
    GetTimeOffset( ) const
    {
      return m_data.timeOffset;
    }
    
    inline FrProcDataData::tRange_type FrProcDataData::
    GetTRange( ) const
    {
      return m_data.tRange;
    }

    inline FrProcDataData::fShift_type FrProcDataData::
    GetFShift( ) const
    {
      return m_data.fShift;
    }

    inline FrProcDataData::phase_type FrProcDataData::
    GetPhase( ) const
    {
      return m_data.phase;
    }

    inline FrProcDataData::fRange_type FrProcDataData::
    GetFRange( ) const
    {
      return m_data.fRange;
    }

    inline FrProcDataData::BW_type FrProcDataData::
    GetBW( ) const
    {
      return m_data.BW;
    }

    inline void FrProcDataData::
    SetFRange( fRange_type FRange )
    {
      m_data.fRange = FRange;
    }

    inline void FrProcDataData::
    SetFShift( fShift_type FShift )
    {
      m_data.fShift = FShift;
    }

    inline void FrProcDataData::
    SetTimeOffset( timeOffset_type time )
    {
      m_data.timeOffset = time;
    }

    inline void FrProcDataData::
    SetTRange( tRange_type TRange )
    {
      m_data.tRange = TRange;
    }

    //===================================================================
    //===================================================================
    class FrProcData
      : public Common::FrameSpec::Object,
        public FrProcDataData,
	public Common::TOCInfo
    {
    public:
      typedef Common::Container< FrVect >	data_type;
      typedef data_type::const_iterator const_iterator;
      typedef data_type::iterator iterator;

      typedef Common::Container< FrVect >	aux_type;
      typedef aux_type::const_iterator const_aux_iterator;
      typedef aux_type::iterator aux_iterator;

      typedef Common::Container< FrTable >	table_type;
      typedef table_type::const_iterator const_table_iterator;
      typedef table_type::iterator table_iterator;

      typedef Common::Container< FrHistory >	history_type;
      typedef history_type::const_iterator const_history_iterator;
      typedef history_type::iterator history_iterator;

      //: Default constructor
      FrProcData( );

      FrProcData( const FrProcData& Source );

      FrProcData( const std::string& Name,
		  const std::string& Comment,
		  INT_2U Type,
		  INT_2U SubType,
		  REAL_8 TimeOffset,
		  REAL_8 TRange,
		  REAL_8 FShift,
		  REAL_4 Phase,
		  REAL_8 FRange,
		  REAL_8 BW );

      FrProcData( const std::string& Name,
		  REAL_8 SampleRate,
		  const GPSTime& TimeOffset,
		  REAL_8 FShift,
		  REAL_4 Phase );
      FrProcData( Previous::FrProcData& Source,
		  istream_type* Stream );

      ~FrProcData( );

      static promote_ret_type
      Promote( INT_2U Source,
	       promote_arg_type Obj,
	       istream_type* Stream );

      static const char* StructName( );

      static const Common::Description* StructDescription( );

      //: Comparison operator
      bool operator==( const FrProcData& RHS ) const;
      bool operator!=( const FrProcData& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream) const;

      // Accessors
      REAL_8 GetFRange( ) const;
      REAL_8 GetTRange( ) const;

      const std::string& GetNameSlow( ) const;


      const aux_type& RefAux( ) const;
      aux_type& RefAux( );
      const data_type& RefData( ) const;
      data_type& RefData( );
      const table_type& RefTable( ) const;
      table_type& RefTable( );
      const history_type& RefHistory( ) const;
      history_type& RefHistory( );
   
      //: Merge with another FrProcData
      FrProcData& Merge( const FrProcData& RHS );

      std::auto_ptr< FrProcData >
      SubFrProcData( REAL_8 Offset, REAL_8 Dt ) const;

      virtual FrProcData* Create( istream_type& Stream ) const;

      virtual const char* ObjectStructName( ) const;

      //: Write the structure to the stream
      virtual void Write( ostream_type& Stream ) const;

      virtual void TOCQuery( int InfoClass, ... ) const;

      static const std::string& IDTypeToString( INT_2U Type );
      static const std::string& IDSubTypeToString( INT_2U Type,
                                                   INT_2U SubType );

    protected:
      /// \brief Down grade an object
      virtual demote_ret_type
      demote( INT_2U Target,
	      demote_arg_type Obj,
	      istream_type* Stream ) const;

      /// \brief Upgrade an object
      virtual promote_ret_type
      promote( INT_2U Target,
	       promote_arg_type Obj,
	       istream_type* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      struct fr_proc_data_ref_type
      {
	data_type		data;
	aux_type		aux;
	table_type		table;
	history_type		history;

        fr_proc_data_ref_type( );
        fr_proc_data_ref_type( istream_type& Stream );

        // istream_type& operator( )( ostream_type& Stream ) const;

	void copy_core( const fr_proc_data_ref_type& Source );

	bool operator==( const fr_proc_data_ref_type& RHS ) const;
      };
      fr_proc_data_ref_type	m_refs;
      bool			m_synced_with_vector;

      FrProcData( istream_type& Stream );

      void copy_core( const FrProcData& Source );
      std::auto_ptr< FrProcData > sub_time_series( REAL_8 Offset,
                                                   REAL_8 Dt ) const;

      void      sync_with_vector( );
    };

    inline void FrProcData::fr_proc_data_ref_type::
    copy_core( const fr_proc_data_ref_type& Source )
    {
      aux = Source.aux;
      table = Source.table;
      history = Source.history;
    }

    inline bool FrProcData::fr_proc_data_ref_type::
    operator==( const fr_proc_data_ref_type& RHS ) const
    {
      return ( ( this == &RHS )
	       || ( ( data == RHS.data )
		    && ( aux == RHS.aux )
		    && ( table == RHS.table )
		    && ( history == RHS.history ) ) );
    }

    inline cmn_streamsize_type FrProcData::AuxParam_type::
    Bytes( const Common::StreamBase& Stream) const
    {
      return
        sizeof( auxParam )
        + auxParamName.Bytes( )
        ;
    }

    inline bool FrProcData::AuxParam_type::
    operator==( const AuxParam_type& RHS) const
    {
#define CMP__(X) ( X == RHS.X )

      return ( ( this == &RHS )
               || ( CMP__( auxParam )
                    && CMP__( auxParamName )
                    ) )
        ;

#undef CMP__
    }

    const INT_2U FR_PROC_DATA_DEFAULT_PROC_DATA_TYPE =
      FrProcData::UNKNOWN_TYPE;
    const INT_2U FR_PROC_DATA_DEFAULT_SUB_TYPE = FrProcData::UNKNOWN_SUB_TYPE;
    const REAL_8 FR_PROC_DATA_DEFAULT_TRANGE = 0.0;
    const REAL_8 FR_PROC_DATA_DEFAULT_FSHIFT = 0.0;
    const REAL_4 FR_PROC_DATA_DEFAULT_PHASE = 0.0;
    const REAL_8 FR_PROC_DATA_DEFAULT_FRANGE = 0.0;
    const REAL_8 FR_PROC_DATA_DEFAULT_BW = 0.0;

    inline bool FrProcData::fr_proc_data_data_type::
    operator!=( const fr_proc_data_data_type& RHS ) const
    {
      return ! ( *this == RHS );
    }

    inline FrProcData::promote_ret_type FrProcData::
    Promote( INT_2U Source,
	     promote_arg_type Obj,
	     istream_type* Stream )
    {
      return Object::PromoteObject< Previous::FrProcData, FrProcData >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* FrProcData::
    StructName( )
    {
      static const char* name = "FrProcData";
      return name;
    }

    inline REAL_8 FrProcData::
    GetFRange( ) const
    {
      if ( ! m_synced_with_vector )
      {
	const_cast< FrProcData* >( this )->sync_with_vector( );
      }
      return FrProcDataData::GetFRange( );
    }

    inline REAL_8 FrProcData::
    GetTRange( ) const
    {
      if ( ! m_synced_with_vector )
      {
	const_cast< FrProcData* >( this )->sync_with_vector( );
      }
      return FrProcDataData::GetTRange( );
    }

    inline const FrProcData::aux_type& FrProcData::
    RefAux( ) const
    {
      return m_refs.aux;
    }

    inline FrProcData::aux_type& FrProcData::
    RefAux( )
    {
      return m_refs.aux;
    }

    inline const FrProcData::data_type& FrProcData::
    RefData( ) const
    {
      return m_refs.data;
    }

    inline FrProcData::data_type& FrProcData::
    RefData( )
    {
      return m_refs.data;
    }

    inline const FrProcData::history_type& FrProcData::
    RefHistory( ) const
    {
      return m_refs.history;
    }

    inline FrProcData::history_type& FrProcData::
    RefHistory( )
    {
      return m_refs.history;
    }

    inline const FrProcData::table_type& FrProcData::
    RefTable( ) const
    {
      return m_refs.table;
    }

    inline FrProcData::table_type& FrProcData::
    RefTable( )
    {
      return m_refs.table;
    }

    inline bool FrProcData::
    operator==( const FrProcData& RHS ) const
    {
      return ( ( this == &RHS )
	       || ( ( m_data == RHS.m_data )
		    && ( m_refs == RHS.m_refs ) ) );
    }

    inline bool FrProcData::
    operator!=( const FrProcData& RHS ) const
    {
      return ! ( *this == RHS );
    }
  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrProcData_HH */
